#
#   Pyrex Top Level
#

import os
import sys
from Scanning import PyrexScanner
import Errors
from Errors import PyrexError, CompileError, InternalError
import Parsing
import Code
from Pyrex.Utils import replace_suffix

if sys.platform == "mac":
  from Pyrex.Mac.MacSystem import c_compile, c_link

class CompilationResult:
  #  c_file         string
  #  listing_file   string
  #  num_errors     integer
  
  c_file = None
  listing_file = None
  object_file = None
  extension_file = None


def extract_module_name(path):
  _, tail = os.path.split(path)
  name, _ = os.path.splitext(tail)
  return name

def parse(source):
  f = open(source, "r")
  s = PyrexScanner(f, source)
  try:
    tree = Parsing.p_module(s)
  finally:
    f.close()
  return tree

def generate(name, tree, output):
  code = Code.Generator(output)
  tree.process(name, code)

def compile(source, 
    keep_files = 0,
    c_only = 0,
    obj_only = 0,
    dump_parse_tree = 0,
    use_listing_file = 1):
  try:
    result = CompilationResult()
    output_generated = 0
    listing = replace_suffix(source, ".lis")
    if use_listing_file:
      Errors.open_listing_file(listing)
    try:
      tree = parse(source)
      if dump_parse_tree:
        Parsing.print_parse_tree(sys.stdout, tree, 0)
      output = replace_suffix(source, ".c")
      name = extract_module_name(source)
      generate(name, tree, output)
      output_generated = 1
    finally:
      Errors.close_listing_file()
  except CompileError:
    pass
  result.num_errors = Errors.num_errors
  if output_generated:
    result.c_file = os.path.join(os.getcwd(), output)
    #result.c_file = full_output_path
  if use_listing_file: # and result.num_errors > 0:
    result.listing_file = os.path.join(os.getcwd(), listing)
  if result.num_errors > 0 or not output_generated:
    return result
  if c_only:
    return result
  result.object_file = c_compile(result.c_file)
  if obj_only:
    return result
  result.extension_file = c_link(result.object_file)
  return result

def main(c_only = 0, use_listing_file = 1):
  any_failures = 0
  for arg in sys.argv[1:]:
    try:
      result = compile(arg, 
        c_only = c_only, 
        use_listing_file = use_listing_file)
      if result.num_errors > 0:
        any_failures = 1
    except PyrexError, e:
      print e
      any_failures = 1
  if any_failures:
    sys.exit(1)


